<?php

require_once 'Dashboard.php';


class Widget extends Dashboard {

	static
	protected $proto = array(

		'name'                  => array(),

		'source'                => array(
			'list'           => array('section', 'seller', 'search'),
		),
		'argument'              => array(),

		'section_order'         => array(
			'list'           => array('name', 'nameDESC', 'price', 'priceDESC', 'rating', 'ratingDESC'),
			'null'           => TRUE,
			'depend'         => array(
				'target'  => 'source',
				'value'   => 'section',
			),
		),
		'seller_order'          => array(
			'list'           => array('name', 'nameDESC', 'price', 'priceDESC'),
			'null'           => TRUE,
			'depend'         => array(
				'target'  => 'source',
				'value'   => 'seller',
			),
		),

		'type'                  => array(
			'list'           => array('slider', 'tile'),
		),

		// type - slider

		'slider_autoplay'       => array(
			'test'           => 'boolean',
			'default'        => TRUE,
			'depend'         => array(
				'target'  => 'type',
				'value'   => 'slider',
			),
		),
		'slider_autoplay_speed' => array(
			'test'           => 'integer:unsigned',
			'default'        => 3000,
			'depend'         => array(
				'target'  => 'type',
				'value'   => 'slider',
			),
		),

		'limit'                 => array(
			'test'           => array('smallint', 'greater_than', 'less_than:501'),
			'default'        => 25,
		),

		'icon_size'             => array(
			'test'           => array('integer:unsigned', 'greater_than'),
			'default'        => 110,
		),
	);


	// список виджетов
	public function index () {

		$app = $this->app;

		$app->stash->widgets = array_reverse($app->store('widgets'));

		$app->render(array(
			'html' => function($app) {
				include 'templates/widget/index.php';
			},
			'json' => $app->stash->widgets(),
		));
	}

	// новый виджет
	public function add () {

		$app     = $this->app;

		$widgets = $app->store('widgets');
		$id      = $app->safe->uuid();

		$data    = array();

		foreach (self::$proto as $key => $val)
			$data[$key] = isset($val['default'])
					?   $val['default']
					:   NULL;

		$data   ['name'] = $this->_add_index($app->dict('Widget'));

		$widgets[$id]    = $data;

		$app->store ('widgets', $widgets);

		$app->render(array(
			'html' => function($app) use ($id) {
				$app->redirect('widget', array('id' => $id));
			},
			'json' => array('id' => $id),
		));
	}

	// форма виджета
	public function form () {

		$app     = $this->app;

		$widgets = $app->store('widgets');
		$id      = $app->args->id;

		if (! isset($widgets[$id])) return FALSE;

		$app->stash->proto  = self::$proto;
		$app->stash->widget = $widgets[$id];

		$app->render(array('html' => function($app) {
			include 'templates/widget/form.php';
		}));
	}

	// обновление настроек
	public function set () {

		$app     = $this->app;

		$widgets = $app->store('widgets');
		$id      = $app->args->id;

		if (! isset($widgets[$id]))
			return FALSE;

		if (  in_array($id, $app->conf->lock_ids()))
			return FALSE;

		$data    = $this->proto();
		$error   = $this->test ($data);

		if   ($error)
			$result = array('error'   => $app->dict($error));
		else {
			$result = array('success' => $app->dict('Saved'));

			$widgets[$id] = $data;

			$app->store('widgets', $widgets);
		}

		foreach ($result  as $key => $val)
			$app->stash->$key =  $val;

		$app ->render(array('json' => $result))
		or
		$this->form  ();
	}

	// удаление
	public function del () {

		$app     = $this->app;

		$widgets = $app->store('widgets');
		$id      = $app->args->id;

		if (! isset($widgets[$id]))
			return FALSE;

		if (  in_array($id, $app->conf->lock_ids()))
			return FALSE;

		unset      ($widgets[$id]);

		$app->store ('widgets', $widgets);

		$app->render(array(
			'html' => function($app) {
				$app->redirect('widgets');
			},
			'json' => array('success' => TRUE),
		));
	}

	// дублирование
	public function duplicate () {

		$app     = $this->app;

		$widgets = $app->store('widgets');
		$id      = $app->args->id;

		if (! isset($widgets[$id])) return FALSE;

		$new_id  = $app->safe->uuid();
		$data    = $widgets[$id];

		$copy    = $app->dict('Copy');

		$data   ['name']  = $this->_add_index(preg_replace(
			"/(?:\s+\-\s+${copy}\s+#\d+)?\$/", " - ${copy}",
			$data['name'], 1
		));

		$widgets[$new_id] = $data;

		$app->store ('widgets', $widgets);

		$app->render(array(
			'html' => function($app) use ($new_id) {
				$app->redirect('widget', array('id' => $new_id));
			},
			'json' => array('id' => $new_id),
		));
	}

	// --- iframe ----------------------------------------------------------

	public function embed () {

		$app      = $this->app;

		$widgets  = $app->store('widgets');
		$id       = $app->args->id;

		if (! isset($widgets[$id])) return FALSE;

		$widget   = $widgets[$id];
		$items    = array();

		$argument = preg_split('/\s*,\s*/', $widget['argument']);
		$argument = $argument[array_rand($argument)];

		if     ($widget['source'] == 'section') {

			$data = $app->cache('goods', array(
				'id_section' => $argument,
				'page'       => 1,
				'rows'       => $widget['limit'],
				'order'      => $widget['section_order'],
			));
			if (isset($data['rows'])) $items = $data['rows'];
		}
		elseif ($widget['source'] == 'seller') {

			$data = $app->cache('seller_goods', array(
				'id_seller'  => $argument,
				'page'       => 1,
				'rows'       => $widget['limit'],
				'order'      => $widget['seller_order'],
			));
			if (isset($data['rows'])) $items = $data['rows'];
		}
		elseif ($widget['source'] == 'search') {

			if (function_exists('mb_strtolower'))
				$argument =  mb_strtolower($argument);

			$data = $app->cache('search', array(
				'query'      => $argument,
				'pagenum'    => 1,
				'pagesize'   => $widget['limit'],
			));
			if (isset($data['items']))
				foreach ($data['items'] as $item) {

					$item ['name']      = $item['name'] [$app->lang_short];
					$item ['price']     = $item['price'][$app->currency];
					$item ['currency']  = $app->currency;

					$items[$item['id']] = $item;
				}
		}

		$app->stash->widget = $widget;
		$app->stash->items  = $items;

		$app->render(array('html' => function($app) {
			include 'templates/widget/embed.php';
		}));
	}

	// ---------------------------------------------------------------------

	private function _add_index ($name) {

		$widgets = $this->app->store('widgets');

		$re      = sprintf('|^%s\s+#(\d+)$|', preg_quote($name));
		$index   = 0;

		foreach ($widgets as $item)
			if (	   preg_match($re, $item['name'], $match)
				&& $match[1] > $index
			)
				$index = $match[1];

		return $name .= ' #' . ++$index;
	}
}

?>
